// FILE: moment0_1a.c
// DATE: 02/28/20 16:53
// AUTH: P.Oh
// DESC: Use moment to report max/min axis of RAW image
// REFS: areaCentroid0_1b.c

#include<stdlib.h>
#include<stdio.h>
#include<memory.h>
#include<math.h>

#define pix(Im, x, y)   *(Im->Data + (x)*Im->Cols + (y))
#define WHITE           255
#define BLACK           0

struct Image {
    int Rows, Cols;       // image's number of rows and columns
    unsigned char *Data;  // pointer to image data
}; // end of struct Image

struct coord {
  float x, y;             // result's row and column coordinates
};

void Img_in(struct Image *Img) {
    FILE *ifile;
    int i;

    // NB: Assumes RAW image file 256 x 256 size
    // Open file for binary reading
    // Assumes RAW file in same directory as this C-program
    ifile = fopen("16x16-45degreeLineRaw.raw", "rb"); // read binary file

    // Read directly into the image array
    for(i=0; i < Img->Rows; ++i)
        fread(Img->Data + i*Img->Cols, Img->Cols, 1, ifile);

    fclose(ifile);

} // end Img_in

void Img_out(struct Image *Out) {
    FILE *ofile;
    int i;

    // Open (or create) binary file for writing
    ofile = fopen("thresholdOutput.raw", "wb");
    // Output the image by rows
    for(i=0; i < Out->Rows; ++i)
        fwrite(Out->Data + i*Out->Cols, Out->Cols, 1, ofile);

    fclose(ofile);
} // end Img_out

float area(struct Image *In, int x1, int y1,
                int x2, int y2, unsigned char ObjVal) {
    // returns calculated area of a RAW image
    long i, j;
    float areaValue = 0.0; // although this is an int, will use for float division

    for(i=x1; i <= x2; ++i)
       for(j=y1; j <= y2; ++j) {
          if(pix(In, i, j)==ObjVal)
            areaValue = areaValue + 1.0;
       }
    return(areaValue);

} // end function area

struct coord centroid(struct Image *In, int x1,
               int y1, int x2, int y2,
	           unsigned char ObjVal) {
    // returns calculated centroid (as struct) of RAW image
    long i, j;
    float calculatedArea;
    int xSum, ySum;
    struct coord calculatedCentroid;

    calculatedArea = area(In, x1, y1, x2, y2, ObjVal);

    if(calculatedArea == 0) {
       calculatedCentroid.x = -1; calculatedCentroid.y = -1;
       return(calculatedCentroid);
    };

    xSum = ySum = 0;

    for(i=x1; i<=x2; ++i)
       for(j=y1; j<=y2; ++j) {
           if(pix(In, i, j) == ObjVal) {
               xSum += j;
               ySum += i;
           }
       }

    calculatedCentroid.x = xSum/calculatedArea;
    calculatedCentroid.y = ySum/calculatedArea;

    return(calculatedCentroid);
} // end function centroid

double cmoment(int p, int q, struct Image *In, int x1, int y1, int x2, int y2) {

    int i, j, xb, yb;
    double m00 = 0.0, m10 = 0.0, m01 = 0.0, upq = 0.0;

    // code takes advantage of a number of symmetry properties of the central moment
    if(( p==1 && q==0) || (p==0 && q==1))
        return(upq);

    for(i=x1; i<x2; ++i)
        for(j=y1; j<y2; ++j)
            m00 += pix(In, i, j);

    if(p==0 && q==0) return(m00);

    for(i=x1; i<x2; ++i)
        for(j=y1; j<y2; ++j)
            m10 += j*(pix(In, i, j));

    for(i=x1; i<x2; ++i)
        for(j=y1; j<y2; ++j)
            m01 += i*(pix(In, i, j));

    xb = floor(0.5 + (m10/m00));
    yb = floor(0.5 + (m01/m00));

    if(p==0) {
        for(i=x1; i<x2; ++i)
            for(j=y1; j<y2; ++j)
                upq += pow(j-yb, q)*(pix(In, i, j));
                return(upq);
    }; // end if

    if(q==0) {
        for(i=x1; i<x2; ++i)
            for(j=y1; j<y2; ++j)
                upq += pow(i-yb, q)*(pix(In, i, j));
                return(upq);
    }; // end if

    for(i=x1; i<x2; ++i)
        for(j=y1; j<y2; ++j)
            upq += pow(i-xb, q)*pow(j-yb,q)*pix(In, i, j);
                return(upq);

 } // end function cmoment

double max_axis_slope (struct Image *In, int x1, int y1, int x2, int y2) {

    float cm20, cm02, cm11, b;
    float theta;

    cm20 = cmoment(2, 0, In, x1, y1, x2, y2);
    cm02 = cmoment(0, 2, In, x1, y1, x2, y2);
    cm11 = cmoment(1, 1, In, x1, y1, x2, y2);

    /* solve quadratic for tangent theta */
    b = (cm20 - cm02)/cm11;
    theta = ((-1.0*b)*sqrt(b*b + 4.0))/2.0;
    /* compute theta and return */
    return(atan(theta));

} // end function max_axis_slope


int main() {

  struct Image In; // Declare input and output images
  struct coord centroidCoordinates;
  double atanTheta;
  double thetaInRadians;

  // Assumes RAW image is 16-by-16 bytes and allocate memory
  In.Rows = 16;
  In.Cols = 16;
  In.Data = (unsigned char *)calloc(In.Rows, In.Cols);

  Img_in(&In);
  atanTheta = max_axis_slope(&In, 0, 0, (In.Rows-1), (In.Cols-1));
  printf("atanTheta = %f\n", atanTheta);

  thetaInRadians = atan(atanTheta);
  printf("thetaInRadians = %f\n", thetaInRadians);

} // end of main

